{
  "registries": [
    {
      "kind": "git",
      "repository": "https://github.com/JonatanNevo/portal-vcpkg-registry",
      "reference": "main",
      "baseline": "{{ registry_ref }}",
      "packages": [{% for port in ports %}"{{ port.name }}"{% if not loop.last %}, {% endif %}{% endfor %}]
    }
  ]
}
